import { is } from 'ramda';

import { Action as ReduxAction, Middleware } from 'redux';
import { ThunkAction, ThunkDispatch } from 'redux-thunk';

import { getAuthenticateAttribute } from '@treejs/auth/selectors';
import { Action } from '@treejs/types/redux/actions';
import { isJSON, isNilOrEmpty } from '@treejs/utils';

import { onError, onRequest, onSuccess } from './slice';
import { ACTIONS, FetchConfig } from './types';
import { resolveParams } from './utils';

type ISettings<C> = {
	BASE_ENDPOINT_URL: string;
	authorization?: {
		authAttribute?: string;
		header?: string;
		schema?: string;
	};
	onError?: (error: any, setting: FetchConfig<C>) => ThunkAction<void, any, unknown, ReduxAction<string>>;
};

type IStore = {
	dispatch: ThunkDispatch<any, unknown, ReduxAction<string>>;
	getState: () => any;
};

const resolveCatch = <C,>(store: IStore, action: Action<FetchConfig<any>>, setting: ISettings<C>, error: any) => {
	store.dispatch(onError(action.payload));
	if (is(Function, setting.onError)) {
		store.dispatch(setting.onError(error, action.payload));
	}
	if (is(Function, action.payload.onError)) {
		action.payload.onError(error);
	}
	return error;
};

export const apiMiddleware = <C,>(setting: ISettings<C>): Middleware => (store: IStore) => (next) => async (
	action: Action<FetchConfig<C>>
) => {
	if (action.type === ACTIONS.FETCH) {
		if (isNilOrEmpty(action.payload.url)) {
			throw `URL '${action.payload.url}' is not valid, please specify valid URL!`;
		}

		if (is(Function, action.payload.onRequest)) {
			await action.payload.onRequest();
		}

		store.dispatch(onRequest(action.payload));

		const baseUrl = action.payload.baseUrl || setting.BASE_ENDPOINT_URL;
		const endpoint = `${baseUrl}${resolveParams(action.payload.url, action.payload.queryParams)}`;

		const tokenValue = getAuthenticateAttribute(
			store.getState(),
			setting?.authorization?.authAttribute || 'accessToken'
		);
		const authorizationToken = !isNilOrEmpty(tokenValue)
			? {
					[setting?.authorization?.header || 'Authorization']: `${
						setting?.authorization?.schema || 'Bearer'
					} ${tokenValue}`,
			  }
			: {};

		try {
			const response = await fetch(endpoint, {
				headers: {
					'Content-Type': 'application/json',
					...authorizationToken,
					...action.payload.headers,
				},
				method: action.payload.method.toUpperCase(),
				body: JSON.stringify(action.payload.body),
			});

			const json = await response.text().then((text: string) => {
				return isJSON(text) ? JSON.parse(text) : text;
			});

			if (!response.ok) {
				return Promise.reject(resolveCatch<C>(store, action, setting, json));
			}

			store.dispatch(onSuccess(action.payload));
			if (is(Function, action.payload.onSuccess)) {
				await action.payload.onSuccess(json);
			}
			return Promise.resolve(json);
		} catch (error) {
			return Promise.reject(resolveCatch<C>(store, action, setting, error));
		}
	}

	return next(action);
};
