import React from 'react';
import { any, compose, has, ifElse, includes, is, isNil, keys, mapObjIndexed, values } from 'ramda';

import cx from 'classnames';

import { isNilOrEmpty } from '@treejs/utils';

import { GRID_SIZE, VERTICAL_ALIGN } from './types';

type IGridProps = {
	children?: React.ReactElement | React.ReactElement[];
	className?: string;
	cols?: Partial<Record<GRID_SIZE, number>> | number;
	space?: number;
	vertical?: VERTICAL_ALIGN;
};

const isBiggerDefined = (cols: Partial<Record<GRID_SIZE, number>> | number): boolean => {
	if (!isNilOrEmpty(cols) && !is(Number, cols)) {
		const sizes = keys(cols);
		let bigger = [];
		if (includes(GRID_SIZE.SM, sizes)) {
			bigger = [GRID_SIZE.MD, GRID_SIZE.LG, GRID_SIZE.XL];
		} else if (includes(GRID_SIZE.MD, sizes)) {
			bigger = [GRID_SIZE.LG, GRID_SIZE.XL];
		} else if (includes(GRID_SIZE.LG, sizes)) {
			bigger = [GRID_SIZE.XL];
		} else {
			return false;
		}
		return any((size) => includes(size, sizes), bigger);
	}
	return true;
};

export const Grid: React.FC<IGridProps> = ({
	cols = 1,
	space = 4,
	className,
	children,
	vertical,
}): React.ReactElement => {
	return (
		<div
			className={cx(
				'grid',
				className,
				{
					'grid-items-start': vertical === VERTICAL_ALIGN.TOP,
					'grid-items-center': vertical === VERTICAL_ALIGN.CENTER,
					'grid-items-end': vertical === VERTICAL_ALIGN.BOTTOM,
				},
				ifElse(
					is(Number),
					() => `grid-cols-${cols}`,
					compose(
						values,
						mapObjIndexed((size, breakpoint) => `${breakpoint}:grid-cols-${size}`)
					)
				)(cols),
				{
					'lg-m:grid-cols-3': isBiggerDefined(cols),
					'md-m:grid-cols-2': isBiggerDefined(cols),
					'sm-m:grid-cols-1': isBiggerDefined(cols),
				},
				{
					[`gap-${space}`]: space !== 0,
				}
			)}
		>
			{children}
		</div>
	);
};

Grid.displayName = 'Grid';

type IGridColProps = {
	children?: React.ReactElement | React.ReactElement[] | string;
	colSpan?: Partial<Record<GRID_SIZE, number>> | number;
	end?: number;
	start?: number;
};

export const GridCol = (props: IGridColProps): React.ReactElement => {
	const { start, end, colSpan, children } = props;
	return (
		<div
			className={cx(
				{
					[`col-start-${start}`]: !isNil(start),
					[`col-end-${end}`]: !isNil(end),
				},
				ifElse(
					is(Number),
					() => `col-span-${colSpan}`,
					compose(
						values,
						mapObjIndexed((size, breakpoint) => `${breakpoint}:col-span-${size}`)
					)
				)(colSpan),
				{
					'sm-m:col-span-1': !has(GRID_SIZE.SM)(colSpan) && !isNil(colSpan),
				}
			)}
		>
			{children}
		</div>
	);
};

GridCol.displayName = 'GridCol';
