import React, { useCallback, useEffect, useMemo, useState } from 'react';

import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import cx from 'classnames';

import { IHTMLElement } from '@treejs/types/form';
import { IOptions } from '@treejs/types/options';
import { isNilOrEmpty } from '@treejs/utils';

import { OptionRenderProps } from '../types';

export type IRadioButtonFieldProps<V = IOptions> = {
	horizontal?: boolean;
	/**
	 * Is called for each item in options
	 */
	optionRender?: React.FunctionComponent<OptionRenderProps>;
	options: IOptions[];
	value?: string;
} & IHTMLElement<V>;

const RadioButton: React.FC<IRadioButtonFieldProps> = (props) => {
	const { onClick, onBlur, horizontal = false, title, options, optionRender } = props;

	const [value, setValue] = useState(null);

	useEffect(() => {
		setValue(props.value);
	}, [props.value]);

	const handleClick = useCallback(
		(option: IOptions): any => (e: React.MouseEvent<HTMLInputElement>): void => {
			if (onClick) {
				onClick(option, e);
			}
			setValue(option.code);
		},
		[]
	);

	const handleBlur = useCallback(
		(option: IOptions): any => (e: React.FocusEvent<HTMLInputElement>): void => {
			if (onBlur) {
				onBlur(option, e);
			}
		},
		[]
	);

	const optionElements = useMemo(() => {
		return options.map(
			({ code, name }): React.ReactElement => (
				<div
					className={cx({
						'mb-2 block': !horizontal,
						'inline-block w-full': horizontal,
					})}
					key={code}
				>
					<input
						className="hidden"
						id={`${props.name}_${code}`}
						name={name}
						type="radio"
						onClick={handleClick({ code, name })}
						onBlur={handleBlur({ code, name })}
					/>
					<label htmlFor={`${props.name}_${code}`} className="cursor-pointer text-primary">
						{!isNilOrEmpty(optionRender) ? (
							React.createElement(optionRender, {
								option: { code, name },
								value,
							})
						) : (
							<>
								<FontAwesomeIcon size="lg" icon={value === code ? 'dot-circle' : 'circle'} />
								<span className="ml-1 text-black">{name}</span>
							</>
						)}
					</label>
				</div>
			)
		);
	}, [value, options, props.name]);

	return (
		<div className="relative mt-1">
			<div className="block text-black pb-2 field-label">{title}</div>
			{horizontal ? <div className="flex">{optionElements}</div> : optionElements}
		</div>
	);
};

export default RadioButton;
