import React, { useCallback, useEffect, useState } from 'react';

import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import cx from 'classnames';

import { STATUS } from '@treejs/types/common';
import { IHTMLElement } from '@treejs/types/form';
import { isNilOrEmpty, masker } from '@treejs/utils';

export type ITextFieldProps<V = string> = {
	autoFocus?: boolean;
	clearable?: boolean;
	defaultValue?: string;
	disabled?: boolean;
	mask?: string;
	readonly?: boolean;
	status?: STATUS;
	type?: 'text' | 'password';
	value?: string;
} & IHTMLElement<V>;

const TextField: React.FC<ITextFieldProps> = (props) => {
	const {
		type = 'text',
		onClick,
		onFocus,
		onBlur,
		onChange,
		disabled,
		name,
		title,
		readonly,
		status,
		autoFocus,
		defaultValue,
		mask,
		clearable = true,
	} = props;

	const inputRef = React.createRef<HTMLInputElement>();

	const [value, setValue] = useState(null);

	const applyMask = useCallback(
		(val) => {
			if (!isNilOrEmpty(mask) && !isNilOrEmpty(val)) {
				return masker.toPattern(val, mask);
			}
			return val;
		},
		[mask]
	);

	useEffect((): void => {
		if (!isNilOrEmpty(defaultValue)) {
			setValue(applyMask(defaultValue));
		}
	}, [defaultValue]);

	useEffect((): void => {
		setValue(applyMask(props.value));
	}, [props.value]);

	const handleClick = useCallback(
		(e: React.MouseEvent<HTMLInputElement>): void => {
			if (onClick) {
				onClick(e.currentTarget.value, e);
			}
		},
		[onClick]
	);

	const handleFocus = useCallback(
		(e: React.FocusEvent<HTMLInputElement>): void => {
			if (onFocus) {
				onFocus(e.currentTarget.value, e);
			}
		},
		[onFocus]
	);

	const handleBlur = useCallback(
		(e: React.FocusEvent<HTMLInputElement>): void => {
			if (onBlur) {
				onBlur(e.currentTarget.value, e);
			}
		},
		[onBlur]
	);

	const handleClearValue = useCallback(
		(e: React.MouseEvent<HTMLInputElement>): void => {
			setValue(null);
			if (onChange) {
				onChange(null, e);
			}
		},
		[onChange]
	);

	const handleChange = useCallback(
		(e: React.ChangeEvent<HTMLInputElement>): void => {
			const val = e.currentTarget.value;
			if (!disabled) {
				setValue(applyMask(val));
				if (onChange) {
					onChange(applyMask(val), e);
				}
			}
		},
		[disabled, onChange]
	);

	return (
		<div className="field-container">
			<label className="field-label" htmlFor={name}>
				{title}
			</label>
			<input
				className={cx('field-input masked', {
					'field-input--info': status === STATUS.info,
					'field-input--success': status === STATUS.success,
					'field-input--warning': status === STATUS.warning,
					'field-input--error': status === STATUS.error,
				})}
				ref={inputRef}
				autoComplete="off"
				id={name}
				name={name}
				type={type}
				disabled={disabled}
				readOnly={readonly}
				value={value || ''}
				onClick={handleClick}
				onFocus={handleFocus}
				onBlur={handleBlur}
				onChange={handleChange}
				autoFocus={autoFocus}
			/>
			{!disabled && clearable && !isNilOrEmpty(value) && (
				<div className="field-icon field-icon--clear" onClick={handleClearValue}>
					<FontAwesomeIcon icon="backspace" />
				</div>
			)}
		</div>
	);
};

export default TextField;
