import React, { useCallback, useEffect, useState } from 'react';

import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { format, isMatch, setHours, setMinutes } from 'date-fns';

import { VERTICAL_ALIGN } from '@treejs/components/Grid/types';
import TextField from '@treejs/components/TextField';
import { isNilOrEmpty } from '@treejs/utils';

import Button from '../../Button';
import { Grid, GridCol } from '../../Grid';

type IProps = {
	onChange: (value: string) => void;
	value: string;
};

const ButtonField: React.FC<{ name: string; onChange: (a: string) => void; title: string; value: string }> = ({
	title,
	name,
	onChange,
	value,
}) => {
	const handleChange = useCallback(
		(newValue): any => {
			onChange(newValue);
		},
		[onChange]
	);

	const increase = useCallback((): any => {
		const newValue = (Number(value) + 1).toString();
		handleChange(newValue);
	}, [value, handleChange]);

	const decrease = useCallback((): any => {
		const newValue = (Number(value) - 1).toString();
		handleChange(newValue);
	}, [value, handleChange]);

	return (
		<Grid cols={4} vertical={VERTICAL_ALIGN.BOTTOM}>
			<GridCol colSpan={1}>
				<Button label={<FontAwesomeIcon icon="minus" />} onClick={decrease} />
			</GridCol>
			<GridCol colSpan={2}>
				<TextField
					name={name}
					title={title}
					clearable={false}
					value={value}
					onChange={handleChange}
					mask="99"
				/>
			</GridCol>
			<GridCol colSpan={1}>
				<Button label={<FontAwesomeIcon icon="plus" />} onClick={increase} />
			</GridCol>
		</Grid>
	);
};

const TimePickerModal: React.FC<IProps> = (props) => {
	const { onChange } = props;

	const [value, setValue] = useState<Date>(new Date());

	useEffect(() => {
		if (isMatch(props.value, 'H:mm')) {
			const today = format(new Date(), 'yyyy/MM/dd');
			setValue(new Date(`${today} ${props.value}`));
		} else {
			setValue(new Date());
		}
	}, [props.value]);

	const handleChangeHours = useCallback(
		(hour) => {
			if (isNilOrEmpty(hour)) {
				hour = 0;
			} else if (Number(hour) > 24 || Number(hour) < 0) {
				hour = 23;
			}
			const newHour = setHours(value, hour);
			setValue(newHour);
			onChange(format(newHour, 'H:mm'));
		},
		[value, onChange]
	);

	const handleChangeMinutes = useCallback(
		(minute) => {
			if (isNilOrEmpty(minute)) {
				minute = 0;
			} else if (Number(minute) > 60 || Number(minute) < 0) {
				minute = 59;
			}
			const newMinute = setMinutes(value, minute);
			setValue(newMinute);
			onChange(format(newMinute, 'H:mm'));
		},
		[value, onChange]
	);

	return (
		<>
			<Grid cols={1}>
				<GridCol>
					<ButtonField name="hour" title="Hour" value={format(value, 'H')} onChange={handleChangeHours} />
				</GridCol>
				<GridCol>
					<ButtonField
						name="minute"
						title="Minute"
						value={format(value, 'm')}
						onChange={handleChangeMinutes}
					/>
				</GridCol>
			</Grid>
		</>
	);
};

export default TimePickerModal;
